package com.flutter_dhl_amap.flutter_dhl_amap.Location;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.util.Log;

import com.amap.api.location.AMapLocation;
import com.amap.api.location.AMapLocationClient;
import com.amap.api.location.AMapLocationClientOption;
import com.amap.api.location.AMapLocationListener;
import com.amap.api.services.core.ServiceSettings;
import com.flutter_dhl_amap.flutter_dhl_amap.Class.ConstraintsMap;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Locale;
import java.util.Map;

import io.flutter.plugin.common.EventChannel;

public class DHL_LocationModel implements AMapLocationListener {
    private AMapLocationClient mLocationClient = null;//定位
    private EventChannel.EventSink eventSink;
    private Context context;

    public   String sHA1(Context context) {
        try {
            PackageInfo info = context.getPackageManager().getPackageInfo(
                    context.getPackageName(), PackageManager.GET_SIGNATURES);
            byte[] cert = info.signatures[0].toByteArray();
            MessageDigest md = MessageDigest.getInstance("SHA1");
            byte[] publicKey = md.digest(cert);
            StringBuffer hexString = new StringBuffer();
            for (int i = 0; i < publicKey.length; i++) {
                String appendString = Integer.toHexString(0xFF & publicKey[i])
                        .toUpperCase(Locale.US);
                if (appendString.length() == 1)
                    hexString.append("0");
                hexString.append(appendString);
                hexString.append(":");
            }
            String result = hexString.toString();
            return result.substring(0, result.length()-1);
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        return null;
    }
    public DHL_LocationModel(Context context, Map<String, Object> data, EventChannel.EventSink eventSink){
        this.eventSink = eventSink;
        this.context = context;
//        String ff = sHA1(context);
        ServiceSettings.updatePrivacyShow(context, true, true);

        ServiceSettings.updatePrivacyAgree(context,true);
        try {
            mLocationClient = new AMapLocationClient(context);
        } catch (Exception e) {
            e.printStackTrace();
        }
        mLocationClient.setLocationListener(this);
        AMapLocationClientOption option = new AMapLocationClientOption();
        parseOptions(option,data);
        mLocationClient.setLocationOption(option);

        mLocationClient.startLocation();
    }
    public void stop_location(){
        mLocationClient.stopLocation();
    }


    private void parseOptions(AMapLocationClientOption option,Map arguments) {
        //  AMapLocationClientOption option = new AMapLocationClientOption();
        Boolean onceLocation = (Boolean) arguments.get("onceLocation");
        option.setLocationMode(AMapLocationClientOption.AMapLocationMode.valueOf((String) arguments.get("locationMode")));//可选，设置定位模式，可选的模式有高精度、仅设备、仅网络。默认为高精度模式
        option.setGpsFirst( (Boolean)arguments.get("gpsFirst") );//可选，设置是否gps优先，只在高精度模式下有效。默认关闭
        option.setHttpTimeOut(  (Integer) arguments.get("httpTimeOut"));//可选，设置网络请求超时时间。默认为30秒。在仅设备模式下无效
        option.setInterval((Integer) arguments.get("interval"));//可选，设置定位间隔。默认为2秒
        option.setNeedAddress((Boolean) arguments.get("needsAddress"));//可选，设置是否返回逆地理地址信息。默认是true
        option.setOnceLocation(onceLocation);//可选，设置是否单次定位。默认是false
        option.setOnceLocationLatest((Boolean) arguments.get("onceLocationLatest"));//可选，设置是否等待wifi刷新，默认为false.如果设置为true,会自动变为单次定位，持续定位时不要使用
        AMapLocationClientOption.setLocationProtocol(AMapLocationClientOption.AMapLocationProtocol.valueOf((String) arguments.get("locationProtocal")));//可选， 设置网络请求的协议。可选HTTP或者HTTPS。默认为HTTP
        option.setSensorEnable((Boolean) arguments.get("sensorEnable"));//可选，设置是否使用传感器。默认是false
        option.setWifiScan((Boolean) arguments.get("wifiScan")); //可选，设置是否开启wifi扫描。默认为true，如果设置为false会同时停止主动刷新，停止以后完全依赖于系统刷新，定位位置可能存在误差
        option.setLocationCacheEnable((Boolean) arguments.get("locationCacheEnable")); //可选，设置是否使用缓存定位，默认为true
        AMapLocationClientOption.GeoLanguage  geoLanguage_type = null;
        switch ((Integer) arguments.get("geoLanguage")) {
            case 0:
                geoLanguage_type = AMapLocationClientOption.GeoLanguage.DEFAULT;
                break;
            case 1:
                geoLanguage_type = AMapLocationClientOption.GeoLanguage.EN;
                break;
            case 2:
                geoLanguage_type = AMapLocationClientOption.GeoLanguage.ZH;
                break;
            default:
                geoLanguage_type = AMapLocationClientOption.GeoLanguage.DEFAULT;
                break;
        }
        option.setGeoLanguage( geoLanguage_type);//可选，设置逆地理信息的语言，默认值为默认语言（根据所在地区选择语言）

    }

    @Override
    public void onLocationChanged(AMapLocation aMapLocation) {
        if(eventSink!=null){
            ConstraintsMap map = new ConstraintsMap();
            map.putString("event","location");
            map.putDouble("lat",aMapLocation.getLatitude());
            map.putDouble("lon",aMapLocation.getLongitude());
            map.putString("formattedAddress", aMapLocation.getAddress());
            map.putString("country", aMapLocation.getCountry());
            map.putString("province", aMapLocation.getProvince());
            map.putString("city", aMapLocation.getCity());
            map.putString("district", aMapLocation.getDistrict());
            map.putString("citycode", aMapLocation.getCityCode());
            map.putString("adcode", aMapLocation.getAdCode());
            map.putString("street", aMapLocation.getStreet());
            map.putString("number", aMapLocation.getStreetNum());
            map.putString("POIName", aMapLocation.getPoiName());
            map.putString("AOIName", aMapLocation.getAoiName());
            eventSink.success(map.toMap());


        }else{
            Log.i("amap","未连接上");
        }
    }
}
